"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "Reference", {
  enumerable: true,
  get: function get() {
    return _reference["default"];
  }
});
Object.defineProperty(exports, "Scope", {
  enumerable: true,
  get: function get() {
    return _scope["default"];
  }
});
Object.defineProperty(exports, "ScopeManager", {
  enumerable: true,
  get: function get() {
    return _scopeManager["default"];
  }
});
Object.defineProperty(exports, "Variable", {
  enumerable: true,
  get: function get() {
    return _variable["default"];
  }
});
exports.analyze = analyze;
Object.defineProperty(exports, "version", {
  enumerable: true,
  get: function get() {
    return _package.version;
  }
});
var _assert = _interopRequireDefault(require("assert"));
var _scopeManager = _interopRequireDefault(require("./scope-manager"));
var _referencer = _interopRequireDefault(require("./referencer"));
var _reference = _interopRequireDefault(require("./reference"));
var _variable = _interopRequireDefault(require("./variable"));
var _scope = _interopRequireDefault(require("./scope"));
var _package = require("../package.json");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }
function defaultOptions() {
  return {
    optimistic: false,
    directive: false,
    nodejsScope: false,
    impliedStrict: false,
    sourceType: 'script',
    // one of ['script', 'module']
    ecmaVersion: 5,
    childVisitorKeys: null,
    fallback: 'iteration'
  };
}
function updateDeeply(target, override) {
  var key, val;
  function isHashObject(target) {
    return _typeof(target) === 'object' && target instanceof Object && !(target instanceof Array) && !(target instanceof RegExp);
  }
  for (key in override) {
    if (override.hasOwnProperty(key)) {
      val = override[key];
      if (isHashObject(val)) {
        if (isHashObject(target[key])) {
          updateDeeply(target[key], val);
        } else {
          target[key] = updateDeeply({}, val);
        }
      } else {
        target[key] = val;
      }
    }
  }
  return target;
}

/**
 * Main interface function. Takes an Esprima syntax tree and returns the
 * analyzed scopes.
 * @function analyze
 * @param {esprima.Tree} tree
 * @param {Object} providedOptions - Options that tailor the scope analysis
 * @param {boolean} [providedOptions.optimistic=false] - the optimistic flag
 * @param {boolean} [providedOptions.directive=false]- the directive flag
 * @param {boolean} [providedOptions.ignoreEval=false]- whether to check 'eval()' calls
 * @param {boolean} [providedOptions.nodejsScope=false]- whether the whole
 * script is executed under node.js environment. When enabled, escope adds
 * a function scope immediately following the global scope.
 * @param {boolean} [providedOptions.impliedStrict=false]- implied strict mode
 * (if ecmaVersion >= 5).
 * @param {string} [providedOptions.sourceType='script']- the source type of the script. one of 'script' and 'module'
 * @param {number} [providedOptions.ecmaVersion=5]- which ECMAScript version is considered
 * @param {Object} [providedOptions.childVisitorKeys=null] - Additional known visitor keys. See [esrecurse](https://github.com/estools/esrecurse)'s the `childVisitorKeys` option.
 * @param {string} [providedOptions.fallback='iteration'] - A kind of the fallback in order to encounter with unknown node. See [esrecurse](https://github.com/estools/esrecurse)'s the `fallback` option.
 * @return {ScopeManager}
 */
function analyze(tree, providedOptions) {
  var scopeManager, referencer, options;
  options = updateDeeply(defaultOptions(), providedOptions);
  scopeManager = new _scopeManager["default"](options);
  referencer = new _referencer["default"](options, scopeManager);
  referencer.visit(tree);
  (0, _assert["default"])(scopeManager.__currentScope === null, 'currentScope should be null.');
  return scopeManager;
}

/* vim: set sw=4 ts=4 et tw=80 : */