"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _es6WeakMap = _interopRequireDefault(require("es6-weak-map"));
var _scope = _interopRequireWildcard(require("./scope"));
var _assert = _interopRequireDefault(require("assert"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
/**
 * @class ScopeManager
 */
var ScopeManager = /*#__PURE__*/function () {
  function ScopeManager(options) {
    _classCallCheck(this, ScopeManager);
    this.scopes = [];
    this.globalScope = null;
    this.__nodeToScope = new _es6WeakMap["default"]();
    this.__currentScope = null;
    this.__options = options;
    this.__declaredVariables = new _es6WeakMap["default"]();
  }
  _createClass(ScopeManager, [{
    key: "__useDirective",
    value: function __useDirective() {
      return this.__options.directive;
    }
  }, {
    key: "__isOptimistic",
    value: function __isOptimistic() {
      return this.__options.optimistic;
    }
  }, {
    key: "__ignoreEval",
    value: function __ignoreEval() {
      return this.__options.ignoreEval;
    }
  }, {
    key: "__isNodejsScope",
    value: function __isNodejsScope() {
      return this.__options.nodejsScope;
    }
  }, {
    key: "isModule",
    value: function isModule() {
      return this.__options.sourceType === 'module';
    }
  }, {
    key: "isImpliedStrict",
    value: function isImpliedStrict() {
      return this.__options.impliedStrict;
    }
  }, {
    key: "isStrictModeSupported",
    value: function isStrictModeSupported() {
      return this.__options.ecmaVersion >= 5;
    }

    // Returns appropriate scope for this node.
  }, {
    key: "__get",
    value: function __get(node) {
      return this.__nodeToScope.get(node);
    }

    /**
     * Get variables that are declared by the node.
     *
     * "are declared by the node" means the node is same as `Variable.defs[].node` or `Variable.defs[].parent`.
     * If the node declares nothing, this method returns an empty array.
     * CAUTION: This API is experimental. See https://github.com/estools/escope/pull/69 for more details.
     *
     * @param {Esprima.Node} node - a node to get.
     * @returns {Variable[]} variables that declared by the node.
     */
  }, {
    key: "getDeclaredVariables",
    value: function getDeclaredVariables(node) {
      return this.__declaredVariables.get(node) || [];
    }

    /**
     * acquire scope from node.
     * @method ScopeManager#acquire
     * @param {Esprima.Node} node - node for the acquired scope.
     * @param {boolean=} inner - look up the most inner scope, default value is false.
     * @return {Scope?}
     */
  }, {
    key: "acquire",
    value: function acquire(node, inner) {
      var scopes, scope, i, iz;
      function predicate(scope) {
        if (scope.type === 'function' && scope.functionExpressionScope) {
          return false;
        }
        if (scope.type === 'TDZ') {
          return false;
        }
        return true;
      }
      scopes = this.__get(node);
      if (!scopes || scopes.length === 0) {
        return null;
      }

      // Heuristic selection from all scopes.
      // If you would like to get all scopes, please use ScopeManager#acquireAll.
      if (scopes.length === 1) {
        return scopes[0];
      }
      if (inner) {
        for (i = scopes.length - 1; i >= 0; --i) {
          scope = scopes[i];
          if (predicate(scope)) {
            return scope;
          }
        }
      } else {
        for (i = 0, iz = scopes.length; i < iz; ++i) {
          scope = scopes[i];
          if (predicate(scope)) {
            return scope;
          }
        }
      }
      return null;
    }

    /**
     * acquire all scopes from node.
     * @method ScopeManager#acquireAll
     * @param {Esprima.Node} node - node for the acquired scope.
     * @return {Scope[]?}
     */
  }, {
    key: "acquireAll",
    value: function acquireAll(node) {
      return this.__get(node);
    }

    /**
     * release the node.
     * @method ScopeManager#release
     * @param {Esprima.Node} node - releasing node.
     * @param {boolean=} inner - look up the most inner scope, default value is false.
     * @return {Scope?} upper scope for the node.
     */
  }, {
    key: "release",
    value: function release(node, inner) {
      var scopes, scope;
      scopes = this.__get(node);
      if (scopes && scopes.length) {
        scope = scopes[0].upper;
        if (!scope) {
          return null;
        }
        return this.acquire(scope.block, inner);
      }
      return null;
    }
  }, {
    key: "attach",
    value: function attach() {}
  }, {
    key: "detach",
    value: function detach() {}
  }, {
    key: "__nestScope",
    value: function __nestScope(scope) {
      if (scope instanceof _scope.GlobalScope) {
        (0, _assert["default"])(this.__currentScope === null);
        this.globalScope = scope;
      }
      this.__currentScope = scope;
      return scope;
    }
  }, {
    key: "__nestGlobalScope",
    value: function __nestGlobalScope(node) {
      return this.__nestScope(new _scope.GlobalScope(this, node));
    }
  }, {
    key: "__nestBlockScope",
    value: function __nestBlockScope(node, isMethodDefinition) {
      return this.__nestScope(new _scope.BlockScope(this, this.__currentScope, node));
    }
  }, {
    key: "__nestFunctionScope",
    value: function __nestFunctionScope(node, isMethodDefinition) {
      return this.__nestScope(new _scope.FunctionScope(this, this.__currentScope, node, isMethodDefinition));
    }
  }, {
    key: "__nestForScope",
    value: function __nestForScope(node) {
      return this.__nestScope(new _scope.ForScope(this, this.__currentScope, node));
    }
  }, {
    key: "__nestCatchScope",
    value: function __nestCatchScope(node) {
      return this.__nestScope(new _scope.CatchScope(this, this.__currentScope, node));
    }
  }, {
    key: "__nestWithScope",
    value: function __nestWithScope(node) {
      return this.__nestScope(new _scope.WithScope(this, this.__currentScope, node));
    }
  }, {
    key: "__nestClassScope",
    value: function __nestClassScope(node) {
      return this.__nestScope(new _scope.ClassScope(this, this.__currentScope, node));
    }
  }, {
    key: "__nestSwitchScope",
    value: function __nestSwitchScope(node) {
      return this.__nestScope(new _scope.SwitchScope(this, this.__currentScope, node));
    }
  }, {
    key: "__nestModuleScope",
    value: function __nestModuleScope(node) {
      return this.__nestScope(new _scope.ModuleScope(this, this.__currentScope, node));
    }
  }, {
    key: "__nestTDZScope",
    value: function __nestTDZScope(node) {
      return this.__nestScope(new _scope.TDZScope(this, this.__currentScope, node));
    }
  }, {
    key: "__nestFunctionExpressionNameScope",
    value: function __nestFunctionExpressionNameScope(node) {
      return this.__nestScope(new _scope.FunctionExpressionNameScope(this, this.__currentScope, node));
    }
  }, {
    key: "__isES6",
    value: function __isES6() {
      return this.__options.ecmaVersion >= 6;
    }
  }]);
  return ScopeManager;
}();
/* vim: set sw=4 ts=4 et tw=80 : */
exports["default"] = ScopeManager;